/*
 *      PCBeep 1.0 - emits a beep on PC hardware lacking VT support
 *      Copyright (c) 2005 - Willy Tarreau <willy@w.ods.org>
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 *
 * Usage: pcbeep [ dur_ms ] [ freq_hz ] [ wait_ms ]
 * 
 * Default: 150 ms, 800 Hz, 30 ms.
 *
 */

#include <stdio.h>
#include <sys/io.h>
#include <time.h>

#define PRINT(a) write(2, a, sizeof(a)-1)

main(int argc, char **argv) {
    int dur, frq, old, del;
    struct timespec req;
    char *err="\0";

    if (argc > 1)
	dur = strtoul(argv[1], &err, 0);
    else
	dur = 150; // 150 ms by default

    if (argc > 2 && *err == 0)
	frq = strtoul(argv[2], &err, 0);
    else
	frq = 800; // 800 HZ by default

    if (argc > 3 && *err == 0)
	del = strtoul(argv[3], &err, 0);
    else
	del = dur/5; // small delay before exiting by default

    if (*err || dur < 0 || frq < 19 || frq > 1193181) {
	//fprintf(stderr, "Usage: %s [ dur_ms ] [ freq_hz ] [ wait_ms ]\n", argv[0]);
	PRINT("Usage: pcbeep [ dur_ms ] [ freq_hz ] [ wait_ms ]\n");
    }
    
    frq = 1193181 / frq;
    req.tv_sec = dur / 1000;
    req.tv_nsec = (dur % 1000) * 1000000;

    if (ioperm(0x61, 1, 1) == -1 || ioperm(0x42, 2, 1) == -1) {
	//perror("ioperm()");
	PRINT("ioperm(): access denied ?\n");
	return 1;
    }
    
    old = inb(0x61);
    outb(0xB6, 0x43); // freq divisor mode
    outb(frq & 0xff, 0x42);
    outb((frq >> 8) & 0xff, 0x42);
    outb(old | 3, 0x61); // speaker on
    nanosleep(&req, NULL);
    outb(old | 0xFC, 0x61); // speaker off
    req.tv_sec = del / 1000;
    req.tv_nsec = (del % 1000) * 1000000;
    nanosleep(&req, NULL);

    return 0;
}
